<?php

declare(strict_types=1);

namespace Drupal\vipps_mobilepay\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\vipps_mobilepay\Entity\VippsMobilepayConnection;

/**
 * Vipps MobilePay Connection Profile form.
 */
final class VippsMobilepayConnectionForm extends EntityForm {

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state): array {

    $form = parent::form($form, $form_state);

    $form['help'] = [
      '#type' => 'markup',
      '#markup' => $this->t('Please enter API keys for your merchant account down below. Find out more <a href=:url>here</a>', [':url' => 'https://developer.vippsmobilepay.com/docs/knowledge-base/api-keys/']),
    ];

    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#maxlength' => 255,
      '#default_value' => $this->entity->label(),
      '#description' => $this->t('This is an internal value that helps you identify each connection profile. This value is only visible to administrators.'),
      '#required' => TRUE,
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $this->entity->id(),
      '#machine_name' => [
        'exists' => [VippsMobilepayConnection::class, 'load'],
      ],
      '#disabled' => !$this->entity->isNew(),
    ];

    $form['market'] = [
      '#type' => 'select',
      '#title' => $this->t('Market'),
      '#default_value' => $this->entity->get('market'),
      '#options' => [
        'NO' => $this->t('Norway'),
        'DK' => $this->t('Denmark'),
        'FI' => $this->t('Finland'),
      ],
      '#required' => TRUE,
      '#description' => $this->t('Cross-boarder operations are currently not supported by Vipps MobilePay. Choose correct market affiliated with the sales unit for which credentials were issues.'),
    ];

    $form['mode'] = [
      '#type' => 'radios',
      '#title' => $this->t('Mode'),
      '#default_value' => $this->entity->get('mode'),
      '#options' => [
        'test' => $this->t('Test'),
        'live' => $this->t('Live'),
      ],
      '#required' => TRUE,
      '#description' => $this->t('Choose whether the API keys were obtained for the test or live environment'),
    ];

    $form['client_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Client ID'),
      '#maxlength' => 255,
      '#default_value' => $this->entity->get('client_id'),
      '#required' => TRUE,
      '#description' => $this->t('You can either use Merchant or Partner Keys in Client ID, Client Secret and Subscription Key fields. If Partner Keys are entered make sure Merchant is not able to see them by revoking permission to access this page.'),
    ];

    $form['client_secret'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Client Secret'),
      '#maxlength' => 255,
      '#default_value' => $this->entity->get('client_secret'),
      '#required' => TRUE,
    ];

    $form['subscription_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Subscription Key'),
      '#maxlength' => 255,
      '#default_value' => $this->entity->get('subscription_key'),
      '#required' => TRUE,
    ];

    $form['merchant_serial_number'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Merchant Serial Number'),
      '#maxlength' => 255,
      '#default_value' => $this->entity->get('merchant_serial_number'),
      '#required' => TRUE,
      '#description' => $this->t('Enter MSN corresponding with the market selected above. If Merchant supports multiple markets you must create Connection Profile for each supported market.'),
    ];

    $form['status'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enabled'),
      '#default_value' => $this->entity->status(),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state): int {
    $result = parent::save($form, $form_state);
    $message_args = ['%label' => $this->entity->label()];
    $this->messenger()->addStatus(
      match($result) {
        \SAVED_NEW => $this->t('Created new %label.', $message_args),
        \SAVED_UPDATED => $this->t('Updated %label.', $message_args),
      }
    );
    $form_state->setRedirectUrl($this->entity->toUrl('collection'));
    return $result;
  }

}
