<?php

namespace Drupal\vipps_mobilepay;

use Drupal\Core\Extension\ModuleExtensionList;
use Drupal\Core\Http\ClientFactory;
use Http\Adapter\Guzzle7\Client as GuzzleClient;
use zaporylie\Vipps\Api\Authorization;
use zaporylie\Vipps\Client;
use zaporylie\Vipps\ClientInterface;

/**
 * Defines a factory for the VippsMobilePay manager.
 */
class VippsMobilePayManagerFactory implements VippsMobilePayManagerFactoryInterface {

  /**
   * The module extension list.
   *
   * @var \Drupal\Core\Extension\ModuleExtensionList
   */
  protected $moduleExtensionList;

  /**
   * The http client factory.
   *
   * @var \Drupal\Core\Http\ClientFactory
   */
  protected $httpClientFactory;

  /**
   * Array of all instantiated Vipps managers.
   *
   * @var \zaporylie\Vipps\ClientInterface[]
   */
  protected $instances = [];

  public function __construct(ModuleExtensionList $module_extension_list, ClientFactory $http_client_factory) {
    $this->moduleExtensionList = $module_extension_list;
    $this->httpClientFactory = $http_client_factory;
  }

  /**
   * {@inheritdoc}
   */
  public function get(VippsMobilepayConnectionInterface $connection): ClientInterface {
    $merchant_serial_number = $connection->get('merchant_serial_number');
    if (!isset($this->instances[$merchant_serial_number])) {
      $this->instances[$merchant_serial_number] = $this->getVippsClient($connection);
      // Authorize.
      $authorization = new Authorization($this->instances[$merchant_serial_number]);
      $authorization->getToken();
    }

    return $this->instances[$merchant_serial_number];
  }

  /**
   * Get Vipps Client.
   */
  protected function getVippsClient(VippsMobilepayConnectionInterface $connection) {

    // Set Drupal version.
    $options['vipps_system_name'] = 'drupal';
    $options['vipps_system_version'] = \Drupal::VERSION;

    // Set plugin version.
    if ($module = $this->moduleExtensionList->getExtensionInfo('vipps_mobilepay')) {
      $options['vipps_system_plugin_name'] = 'vipps_mobilepay';
      $options['vipps_system_plugin_version'] = $module['version'] ?? 'unknown';
    }

    $client = new Client(
      $connection->get('client_id'),
      $connection->get('client_secret'),
      $connection->get('subscription_key'),
      $connection->get('merchant_serial_number'),
      $options + [
        'http_client' => new GuzzleClient($this->httpClientFactory->fromOptions()),
        'endpoint' => $connection->get('mode') === 'live' ? 'live' : 'test',
      ]);
    return $client;
  }

}
