<?php

declare(strict_types=1);

namespace Drupal\vipps_mobilepay;

use Drupal\Core\Config\Entity\ConfigEntityListBuilder;
use Drupal\Core\Entity\EntityInterface;

/**
 * Provides a listing of Vipps MobilePay connection profiles.
 */
final class VippsMobilepayConnectionListBuilder extends ConfigEntityListBuilder {

  /**
   * {@inheritdoc}
   */
  public function buildHeader(): array {
    $header['label'] = $this->t('Label');
    $header['id'] = $this->t('Machine name');
    $header['market'] = $this->t('Market');
    $header['mode'] = $this->t('Mode');
    $header['status'] = $this->t('Status');
    return $header + parent::buildHeader();
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity): array {
    /** @var \Drupal\vipps_mobilepay\VippsMobilepayConnectionInterface $entity */
    $row['label'] = $entity->label();
    $row['id'] = $entity->id();
    switch ($entity->get('market')) {
      case 'NO':
        $row['market'] = $this->t('Norway');
        break;

      case 'DK':
        $row['market'] = $this->t('Denmark');
        break;

      case 'FI':
        $row['market'] = $this->t('Finland');
        break;
    }
    $row['mode'] = $entity->get('mode') === 'live' ? $this->t('Live') : $this->t('Test');
    $row['status'] = $entity->status() ? $this->t('Enabled') : $this->t('Disabled');

    return $row + parent::buildRow($entity);
  }

}
